import sys

sys.stdin.reconfigure(encoding='utf-8')
sys.stdout.reconfigure(encoding='utf-8')

if len(sys.argv) != 2:
    print("This program expects a filename as input parameter")
    quit()

infilename = sys.argv[1]

def twocomplement(val, bitcount):
    if val & (1 << bitcount - 1):
        val ^= (1 << bitcount) - 1
        val += 1
        val *= -1
    return val

def decode_message(data):
    if len(data) != 5:
        return

    msgtypes = {
        0x04AF : "inside temperature",
        0x05AF : "set temperature",
        0x05FA : "set temperature",
        0x05FF : "set temperature",
        0x06AF : "confirmation set temperature",
        0x06FA : "confirmation set temperature",
        0x09AF : "daily program",
        0x09FA : "daily program",
        0x11AF : "set time/current time",
        0x02F0 : "get time/current time",
        0x60EF : "temperatur sensors ECA86",
        0x01F0 : "outside temperature",
        0x62FE : "set Relays on ECA 86",
    }

    if data[0] in msgtypes:
        print("    %s" % msgtypes[data[0]])

    checksum = 0
    for i in range(4):
        checksum += (data[i] >> 8)# & 0xFF
        checksum += (data[i] >> 0)# & 0xFF
    checksum = checksum & 0xFF
    checksum_sent = data[4] & 0x00FF
    checksum_status = "NOT OK!"
    if checksum == checksum_sent:
        checksum_status = "ok"
    print("    checksum: %02X -> %s" % (checksum, checksum_status))

    

    if data[0] == 0x60EF:
        t1 = float(data[1]) / 128
        t2 = float(data[2]) / 128
        i1 = (data[3] >> 4) & 0x0F
        i2 = (data[3] >> 0) & 0x0F
        print("    T%u: %f °C" % (i1, t1))
        print("    T%u: %f °C" % (i2, t2))
    elif data[0] == 0x04AF:
        t = float(data[1]) / 128
        print("    Ti: %f °C" % t)
    elif data[0] == 0x01F0:
        t = float(data[1]) / 128
        mode_water = (data[2] >> 12) & 0x03
        mode_heat = (data[2] >> 8) & 0x03

        modes = ["Reduced", "Optimized heatup", "Comfort", "Optimized reduced"]
        
        print("    Ta: %f °C" % t)
        print("    Water: %s (%u)" % (modes[mode_water], mode_water))
        print("    Heat: %s (%u)" % (modes[mode_heat], mode_heat))

    elif data[0] in [0x02F0, 0x11AF]:
        min = (data[1] >> 8) & 0x7F
        sec = (data[1] >> 0) & 0x7F
        day = (data[2] >> 8) & 0x3F
        hour = (data[2] >> 0) & 0x3F
        year = (data[3] >> 0) & 0xFF
        weekday = (data[3] >> 12) & 0x0F
        month = (data[3] >> 8) & 0x0F

        weekday_s = ["-", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"][weekday]

        print("    %s, %04u-%02u-%02u %02u:%02u:%02u" % (weekday_s, year + 1900, month, day, hour, min, sec))
    elif data[0] == 0x09AF:
        weekday = (data[1] << 0) & 0x07
        weekday_s = ["Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"][weekday]
        print("    Request day program for %s" % weekday_s)
    elif data[0] == 0x62FE:
        print("    Set Relays: 0x%04X%04X%04X" % (data[1], data[2], data[3]))
    elif data[0] == 0x09FA:
        print("     21:00 18:00 15:00 12:00 09:00 06:00 03:00 00:00")
        print("         |     |     |     |     |     |     |     |")
        print("    {:08b}{:08b}{:08b}{:08b}{:08b}{:08b}".format(
            (data[1] >> 0) & 0xFF,
            (data[1] >> 8) & 0xFF,
            (data[2] >> 0) & 0xFF,
            (data[2] >> 8) & 0xFF,
            (data[3] >> 0) & 0xFF,
            (data[3] >> 8) & 0xFF,
            ))
    elif data[0] == 0x05AF:

        set_temp = (data[1] >> 9) & 0x1F
        print("    set temp: %u °C" % set_temp)

        mode = (data[3] >> 8) & 0x07
        modes = ["Manual", "Automatic", "Comfort", "Reduced", "Standby"]
        mode_s = "!!Unknown!!"
        if mode < len(modes):
            mode_s = modes[mode]


        tmpdiff_relax = twocomplement((data[2] >> 9) & 0x7F, 6)
        tmpdiff_away = twocomplement((data[3]) & 0x7F, 7)

        print("    Tempdiff relax: %s%u °C%s" % 
        (
            "+" if tmpdiff_relax > 0 else "", 
            tmpdiff_relax, 
            " (active)" if mode == 2 else ""
        ))
        print("    Tempdiff away: %s%u °C%s" % 
        (
            "+" if tmpdiff_away > 0 else "", 
            tmpdiff_away, 
            " (active)" if mode == 3 else ""
        ))

        print("    Unknown value: 0x%02X" % (data[2] & 0xFF))

        print("    Mode: %s (%u)" % (mode_s, mode))

        print("    Override active: %u" % (data[3] >> 15))
    else:
        print("    no decoder available")

with open(infilename) as infile:
    lineno = 0

    for line in infile:
        lineno += 1
        
        if lineno == 1:
            #ignore header
            continue

        parts = line.strip("\n").split("\t")

        if len(parts) != 7:
            print("wrong data length @ %u" % lineno)
            continue
        
        time = parts[0]
        data = [int(x, 16) for x in parts[2:2+5]]
        print("message @ %s [%s]:" % (time, " ".join(parts[2:2+5])))
        decode_message(data)
        
